\name{monocar.hist}
\alias{monocar.hist}
\title{Estimate a series history from a MONOCAR model}
\description{
  Estimate the history of a latent process from a Multivariate
  overlapping noisy observation continuous-time autoregressive (MONOCAR)
  model
}
\usage{
monocar.hist(data, model, estimates=NULL, exovars=NULL, byexo="series",
  delta.group=NULL, verbose=1, mu.by.process=FALSE,
  delta.by.process=FALSE, var.by.house=FALSE, \dots)
}
\arguments{
  \item{data}{A data file of class \code{ct.data.frame} (see
    \code{\link{create.ctdata}} and \code{\link{join.ctdata}})}
  \item{model}{A model of class \code{monocar} generated by
    \code{\link{monocar.estimate}}. Can be \code{NULL}, in which case
    the specification is taken from \code{estimates} and
    \code{exovars}.}
  \item{estimates}{If \code{model} is \code{NULL}, a list of
    parameters.}
  \item{exovars}{If \code{model} is \code{NULL}, an optional formula
    giving the exogenous variables to
    be included, a character vector giving the names of exogenous
    variables to be included from \code{data}, or an integer vector
    giving the location of the exogenous variables to be included from
    \code{data}. The default is to include no exogenous variables.}
  \item{byexo}{If \code{model} is \code{NULL}, a string or null value
    specifying whether exogenous
    variables are to have a single coefficient, separate coefficients
    for each series, or separate coefficients for each house.
    Possible values are \code{series}, \code{house} or
    \code{NULL}. Default is \code{NULL}, meaning a single coefficient,
    common to all series and houses, for each exogenous variable.}
  \item{delta.group}{An optional vector of integers specifying which
    houses are associated with which series.}
  \item{verbose}{An integer specifying the amount of output. \code{0}
    means no output, \code{1} means a dot (\code{.}) for each gradient
    computation and a star (\code{*}) for each hessian evaluation, and a
    \code{2} means a dot for each likelihood evaluation, a colon
    (\code{:}) for each gradient evaluation, and a star for each hessian
    evaluation. The default is \code{1}.}
  \item{mu.by.process}{If multiple processes are specified, a boolean
    indicating whether the series means should be identical across
    processes. The default is \code{FALSE}.}
  \item{delta.by.process}{If multiple processes are specified, a boolean
    indicating whether the house means should be identical across
    processes. The default is \code{FALSE}.}
  \item{var.by.house}{Whether variance-transformation parameters should
    be applied by house or by series. The default is to apply by
    series.}
  \item{\dots}{Additional parameters to be passed to the
    \code{monocar.cal}, which carries out the estimation.}
}
\references{Tahk, A. M. (forthcoming) \dQuote{A Continuous-Time,
  Latent-Variable Model of Time-Series Data,} \emph{Political
    Analysis}.}
\seealso{
  \code{\link{monocar.estimate}}, \code{\link{create.ctdata}}, and
  \code{\link{join.ctdata}}.
}
\examples{
}
\keyword{monocar, ts}
