\name{monocar.simulate}
\alias{monocar.simulate}
\alias{simulate.monocar}
\alias{simulate.monocar.ctdata}
\alias{simulate.monocar.latent}
\title{Simulate continuous-time data sets}
\description{Generate simulated continuous-time data sets}
\usage{
simulate.monocar.ctdata(init, var, t1, t2=NULL, data=NULL,
  series.name = NULL, house.name = NULL, process.name = NULL,
  transform = function(x, v) x, transform.var = function(x, v) v,
  exo.data=NULL, exovars=NULL, byexo=NULL, var.centers=NULL)

simulate.monocar.latent(init, time.indices=NULL,
  n.series=NULL, n.processes=NULL,
  data=NULL, exovars=NULL, byexo=NULL, var.centers=NULL)
}
\arguments{
  \item{init}{A an object of class \code{monocar} or a list specifying
  the parameters of the model.}
  \item{var}{A numeric vector giving the variance for each observation.}
  \item{t1}{A numeric vector giving the start time for each observation.}
  \item{t2}{An optional numeric vector giving the end time for each
  observation. By default, the end time of each observation equals the
  start time.}
  \item{data}{An optional data frame or object of class
  \code{ct.data.frame}.}
  \item{series.name}{An optional character, factor, or integer vector
    identifying the series for each observation. The default value of
    \code{NULL} uses the quoted \code{x} argument.}
  \item{house.name}{An optional character, factor, or integer vector
    identifying the house for each observation. The default is to
    use the series.}
  \item{process.name}{An optional character, factor, or integer vector
    identifying the process for each observation. The default is to
    assume all observations belong to the same process.}
  \item{transform}{An optional function specifying a transformation to
    apply to each observation. See \code{\link{monocar.data}} for more
    details.}
  \item{transform.var}{An optional function specifying a transformation
    to apply to the variance of each observation. See
    \code{\link{monocar.data}} for more details.}
  \item{exo.data}{A vector, data frame, matrix, or formula of possible
    exogenous variables to be included in the data.}
  \item{exovars}{An optional formula giving the exogenous variables to
    be included, a character vector giving the names of exogenous
    variables to be included from \code{data}, or an integer vector
    giving the location of the exogenous variables to be included from
    \code{data}. The default is to include no exogenous variables.}
  \item{byexo}{A string or null value specifying whether exogenous
    variables are to have a single coefficient, separate coefficients
    for each series, or separate coefficients for each house.
    Possible values are \code{series}, \code{house} or
    \code{NULL}. Default is \code{NULL}, meaning a single coefficient,
    common to all series and houses, for each exogenous variable.}
  \item{var.centers}{For each house, the value around which to recenter
    the variances. See \code{\link{monocar.cal}} for more
    details.}
  \item{n.series}{An optional integer specifying the number of
    series. This will be inferred from \code{init} or \code{data} if
    available.}
  \item{n.processes}{An optional integer specifying the number of
    processes. This will be inferred from \code{init} or \code{data} if
    available.}
}
\value{
  For \code{simulate.monocar.ctdata}, an object of class \code{ct.data.frame} containing the data.
  For \code{simulate.monocar.latent}, a list containing instantaneous
  observations (in \code{instantaneous.sample} with times given by
  \code{time.indices}) and  non-instantaneous observations (in
  \code{noninstantaneous.sample} with times given by
  \code{interval.start} and \code{interval.end}.
}
\seealso{
  \code{\link{create.ctdata}}, \code{\link{monocar.estimate}}, and \code{\link{monocar.hist}}.
}
\keyword{monocar, datagen, ts}
